package com.jkcredit.record.storage.service.impl;

import com.alibaba.fastjson.JSON;
import com.jkcredit.record.storage.constants.CommonConstant;
import com.jkcredit.record.storage.model.CommonResponseObject;
import com.jkcredit.record.storage.model.MonthResult;
import com.jkcredit.record.storage.service.RecordStorageService;
import com.jkcredit.record.storage.util.MD5Util;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.elasticsearch.action.bulk.BulkItemResponse;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.xcontent.XContentType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author xusonglin
 * @version V1.0
 **/
@Slf4j
@Service
public class RecordStorageServiceImpl implements RecordStorageService {
    @Autowired
    private RestHighLevelClient esRestClient;

    @Override
    public CommonResponseObject recordStorage(List<MonthResult> monthResultList) {
        log.info("recordStorageCount:{}", monthResultList.size());

        BulkRequest bulkRequest = new BulkRequest();

        // 发送es失败id集合
        List<String> errorIds = new ArrayList<>();

        //保存的数据 这里指定json类型
        for (MonthResult monthResult : monthResultList) {
            String id = MD5Util.MD5(monthResult.getMonth() + "_" + monthResult.getVehicleid());
            if (StringUtils.isBlank(id)) {
                id = new SimpleDateFormat("yyyyMMddHHmmssSSS").format(new Date()) + UUID.randomUUID().toString();
            }
            //设置保存的索引和id
            String indexName = "index-record-" + monthResult.getMonth();
            IndexRequest indexRequest = new IndexRequest(indexName).id(id);
            indexRequest.source(JSON.toJSONString(monthResult), XContentType.JSON);
            bulkRequest.add(indexRequest);
            monthResult.setId(id);
        }
        BulkResponse response = null;
        long startTime = System.currentTimeMillis();
        try {
            //执行保存 index同步 indexAsync异步
            response = esRestClient.bulk(bulkRequest, RequestOptions.DEFAULT);
        } catch (IOException ioe) {
            log.error("costTime:{},pushEsException:", (System.currentTimeMillis() - startTime), ioe);
        }
        // 发送es异常
        if (response == null) {
            CommonResponseObject responseObject = new CommonResponseObject();
            responseObject.setCode(CommonConstant.ERROR_CODE_PUSH_ERROR);
            responseObject.setMessage(JSON.toJSONString(monthResultList));
            return responseObject;
        }

        log.info("pushEsResult:{}, costTime:{}", !response.hasFailures(), (System.currentTimeMillis() - startTime));
        // 判断发送es是否存在失败
        if (response.hasFailures()) {
            log.error("pushEsError-errorCount:{}, errorMessage:{}",
                    response.getItems().length, response.getItems()[0].getFailureMessage());
            // 将发送es失败数据的id存入集合
            while (response.iterator().hasNext()) {
                BulkItemResponse bulkItemResponse = response.iterator().next();
                if (bulkItemResponse.isFailed()) {
                    errorIds.add(bulkItemResponse.getId());
                }

            }
        }

        // 存在发送失败数据，将发送失败数据找出并返回
        if (errorIds.size() != 0) {
            Map<String, MonthResult> tempMonthMap = new HashMap<>();
            List<MonthResult> errorList = new ArrayList<>();
            for (MonthResult monthResult : monthResultList) {
                tempMonthMap.put(monthResult.getId(), monthResult);
            }
            for (String errorId : errorIds) {
                MonthResult errorMonthResult = tempMonthMap.get(errorId);
                errorList.add(errorMonthResult);
            }
            CommonResponseObject responseObject = new CommonResponseObject();
            responseObject.setCode(CommonConstant.ERROR_CODE_PUSH_ERROR);
            responseObject.setMessage(JSON.toJSONString(errorList));
            return responseObject;
        } else {
            return new CommonResponseObject().success();
        }
    }
}
